#include "sensor.h"
#include "i2c.h"

extern void tdelay_ms(uint16_t msek);

/* Defines */
#define I2C_ADDR    0x70

#define CMD_INIT    0xe1
#define CMD_MEASURE 0xac
#define CMD_SRESET  0xba

#define STATUS_BUSY 0x80
#define STATUS_CMD  0x40
#define STATUS_CYC  0x20
#define STATUS_CAL  0x04

aht10_st_t AHT10_Init(void)
{
  i2c_status_t res;

  I2C_Start();

  res = I2C_WriteByte(I2C_ADDR);
  if (res != I2C_Ret_OK) {
    return AHT10_St_Err;
  }

  res = I2C_WriteByte(CMD_INIT);
  if (res != I2C_Ret_OK) {
    return AHT10_St_Err;
  }

  I2C_Stop();

  return AHT10_St_OK;
}

aht10_st_t AHT10_SoftReset(void)
{
  i2c_status_t res;

  I2C_Start();

  res = I2C_WriteByte(I2C_ADDR);
  if (res != I2C_Ret_OK) {
    return AHT10_St_Err;
  }

  res = I2C_WriteByte(CMD_SRESET);
  if (res != I2C_Ret_OK) {
    return AHT10_St_Err;
  }

  I2C_Stop();

  return AHT10_St_OK;
}

aht10_st_t AHT10_StartMeasure(void)
{
  i2c_status_t res;

  I2C_Start();

  res = I2C_WriteByte(I2C_ADDR);
  if (res != I2C_Ret_OK) {
    return AHT10_St_Err;
  }

  res = I2C_WriteByte(CMD_MEASURE);
  if (res != I2C_Ret_OK) {
    return AHT10_St_Err;
  }

  I2C_Stop();

  return AHT10_St_OK;
}

aht10_st_t AHT10_GetData(aht10_t * data)
{
  i2c_status_t res;

  I2C_Start();

  /* Send I2C read addr */
  res = I2C_WriteByte(I2C_ADDR | 0x1);
  if (res != I2C_Ret_OK) {
    return AHT10_St_Err;
  }

  /* Now read the value with NACK */
  uint8_t buf[6];
  I2C_ReadByte(&buf[0], 1);
  I2C_ReadByte(&buf[1], 1);
  I2C_ReadByte(&buf[2], 1);
  I2C_ReadByte(&buf[3], 1);
  I2C_ReadByte(&buf[4], 1);
  res = I2C_ReadByte(&buf[5], 0);
  if (res != I2C_Ret_OK) {
    return AHT10_St_Err;
  }

  I2C_Stop();

  data->Status = buf[0];

  if ((buf[0] & STATUS_BUSY) != 0) {
    return AHT10_St_Bsy;
  }

  /* Calculate values */
  uint32_t result;

  /* Humidity = Srh * 100% / 2^20 */
  result = buf[1];
  result <<= 8;
  result |= buf[2];
  result <<= 8;
  result |= buf[3];
  result >>= 4;
  result *= 1000;
  result /= 256;
  result /= 256;
  data->Humidity = (uint16_t)(result / 16);

  int32_t result2;
  /* Temperature = St * 200 / 2^20 - 50 */
  result2 = buf[3] & 0xf;
  result2 <<= 8;
  result2 |= buf[4];
  result2 <<= 8;
  result2 |= buf[5];
  result2 *= 2000;
  result2 /= 256;
  result2 /= 256;
  result2 /= 16;
  data->Temperature = (int16_t)(result2 - 500);

  return AHT10_St_OK;
}
